#!/usr/bin/env python
"""
OpenGL tween.
Using an orthographic projection for 2D graphics.
"""
import os
import pygame
from pygame.locals import *
from OpenGL.GL import *
#from OpenGL.GLU import *
import time
import tween # TODO: from rats import tween

FPS = 60

def resize((width, height)):
    """
    Called when we resize the window.
    (fullscreen on/off)
    """
    if height == 0:
        height = 1
    #glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    # OLD : gluPerspective(45, 1.0 * width / height, 0.1, 100.0) # 45 degrees camera (like a human)
    # NOTE : gluOrtho2D sets up a two-dimensional orthographic viewing region. This is equivalent to calling glOrtho with near=-1 and far=1.
    glOrtho(-1.333333, 1.333333, -1.0, 1.0, -1.0, 1.0)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

def gl_init():
    """
    Init the window
    """
    #glEnable(GL_TEXTURE_2D)
    glShadeModel(GL_SMOOTH)
    glClearColor(0.0, 0.0, 0.0, 0.0) # black background
    #glClearDepth(1.0)
    #glEnable(GL_DEPTH_TEST)
    #glDepthFunc(GL_LEQUAL)
    #glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)

def draw_square():
    glBegin(GL_QUADS)
    glVertex2f(-1.0, -1.0) # Bottom Left of Quad
    glVertex2f( 1.0, -1.0) # Bottom Right of Quad
    glVertex2f( 1.0,  1.0) # Top Right Of Quad
    glVertex2f(-1.0,  1.0) # Top Left Of Quad
    glEnd()

class Test(object):
    size = (1024, 768)
    def __init__(self, **argd):
        self.__dict__.update(**argd)
        # create a display surface. standard pygame stuff
        self.screen = pygame.display.set_mode(self.size, OPENGL | DOUBLEBUF | HWSURFACE)
        pygame.display.set_caption("OpenGL Tween with RATS")
        resize(self.size)
        gl_init()
        self.clock = pygame.time.Clock()
        # tween 
        self.shape_position = 0.0
        self.tweener = None
        self.start_tween()

    def draw(self):
        """
        Renders the screen.
        Called on every frame rendering
        """
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT) # leave this line here
        #-------------------
        #glLoadIdentity()
        #glTranslatef(0.0, 0.0, -5.0)
        #glOrtho(0, 1, 0, 1, -1, 1) # aalex just added this
        current_time = time.time()
        self.shape_position = self.tweener.tick(current_time)
        if self.tweener.is_done(current_time):
            self.start_tween()
        #print("current time: %f" % (current_time))
        #print("position: %f" % (self.shape_position))
        glPushMatrix()
        glTranslate(self.shape_position * 1.33333, self.shape_position * 1.0, 0.0)
        glScale(0.1, 0.1, 0.1)
        glColor(1.0, 0.8, 0.2, 1.0)
        draw_square()
        glPopMatrix()
        # ------------------
        pygame.display.flip() # leave this line here

    def start_tween(self):
        """
        Starts the motion tweening test
        """
        print("Starting tween.")
        try:
            kind = self.tweener.kind
        except AttributeError:
            kind = 0
        start_val = 1.0
        end_val = -1.0 
        duration = 2.0
        current_time = time.time()
        self.tweener = tween.Tween(start_val, end_val, duration, current_time, kind)
        print self.tweener.__dict__
        #for t in range(11):
        #    print self.tween.tick(t)

    def next_tween(self):
        #print ("TODO: switch to next interpolation kind")#self.tween.kind =  
        self.tweener.kind = (self.tweener.kind + 2) % 7 # tween.MAX_TWEEN_KIND_ID
        print "TWEEN style:", tween.kinds[self.tweener.kind], self.tweener.kind
    
    def main(self):
        global FPS
        going = True
        while going:
            events = pygame.event.get()
            for e in events:
                if e.type == QUIT or (e.type == KEYDOWN and e.key == K_ESCAPE):
                    going = False
                elif e.type == KEYDOWN:
                    if e.key == K_f:
                        print("toggle fullscreen")
                        pygame.display.toggle_fullscreen()
                        # resize ?
                    elif e.key == K_RIGHT:
                        self.next_tween()

            self.clock.tick(FPS) # sleeps to reach target FPS and return ms spent
            self.draw()
            #print "FPS:", self.clock.get_fps()

if __name__ == '__main__':
    print("Press ESC to quit.")
    pygame.init()
    Test().main()
    pygame.quit()
